using System.Text.Json.Serialization;

namespace Luski.Classes;

public class Settings
{
    [JsonInclude]
    [JsonPropertyName("scale")]
    public double? Scale { get; set; } = null;
    [JsonInclude]
    [JsonPropertyName("perscrollpixels")]
    public uint PerScrollPixels { get; set; } = 20;
    [JsonInclude]
    [JsonPropertyName("multithreadpercent")]
    public uint MultiThreadPercent { get; set; } = 50;
    [JsonInclude]
    [JsonPropertyName("loadperchannel")]
    public int LoadPerChannel { get; set; } = 50;
    [JsonInclude]
    [JsonPropertyName("theme")]
    public string Theme { get; set; } = "Dark";
    [JsonInclude]
    [JsonPropertyName("experiments")]
    public List<ExperimentJson> Experiments { get; set; } = Array.Empty<ExperimentJson>().ToList();
    [JsonInclude]
    [JsonPropertyName("default_display")]
    public int Display { get; set; } = 0;
    /// <summary>
    /// Sets the log value for the console. Default value of -25 to enable all logs by default except for DrawFrames and InfoOpenGL even if new ones are added.
    /// </summary>
    [JsonInclude]
    [JsonPropertyName("log")]
    public ConsoleLog Logs { get; set; } = (ConsoleLog)(-25);
    [JsonInclude]
    [JsonPropertyName("scale_fonts")]
    public bool ScaleFonts { get; set; } = true;
    [JsonInclude]
    [JsonPropertyName("default_font_px")]
    public uint DefaultFontPX { get; set; } = 20;
    [JsonInclude]
    [JsonPropertyName("top_time_font_px")]
    public uint TopTimeFonttPX { get; set; } = 12;
    [JsonInclude]
    [JsonPropertyName("message_font_px")]
    public uint MessageFontPX { get; set; } = 17;
    [JsonInclude]
    [JsonPropertyName("message_font_line_space_px")]
    public uint MessageFontLineSpacePX { get; set; } = 5;

}

[JsonSerializable(typeof(Settings))]
[JsonSourceGenerationOptions(
    GenerationMode = JsonSourceGenerationMode.Default,
    PropertyNamingPolicy = JsonKnownNamingPolicy.Unspecified,
    WriteIndented = true,
    DefaultIgnoreCondition = JsonIgnoreCondition.Never)]
internal partial class SettingsContext : JsonSerializerContext
{

}